@@ -40,6 +40,9 @@
 #include <inttypes.h>
 #include <sys/types.h>
 
+#ifdef __cplusplus
+extern "C" {
+#endif
 
 /**
  * librdkafka version
@@ -52,7 +55,7 @@
  *
  * I.e.: 0x00080100 = 0.8.1
  */
-#define RD_KAFKA_VERSION  0x00080500
+#define RD_KAFKA_VERSION  0x00080600
 
 /**
  * Returns the librdkafka version as integer.
@@ -1159,3 +1162,6 @@ int rd_kafka_thread_cnt (void);
 int rd_kafka_wait_destroyed (int timeout_ms);
 
 
+#ifdef __cplusplus
+}
+#endif
